
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.event.*;
import javax.swing.event.DocumentEvent.*;

import javax.swing.plaf.*;
import javax.swing.text.Document;
import javax.swing.text.AbstractDocument.Content;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.Element;
import javax.swing.text.BadLocationException;
import javax.swing.text.Segment;
import javax.swing.text.Position;
import javax.swing.text.AttributeSet;
import javax.swing.undo.*;
import java.util.*;

public class MyAbstractDocument implements Document 
{

  Map properties = new TreeMap();
  

  MyContent content;
  MyElement defaultRoot;
  List listeners = new ArrayList();
  
  {
    content = new MyContent();
    defaultRoot = new MyElement(this, 0, content.length());
    
    /* The bounds of the default root is ignored. Only the children are
       examined by the plain view. */

    Element element = new MyElement(this, 0, content.length());
    defaultRoot.subElements.add(element);
    

  }

  /* This method is called by default caret right after the document
     is set on a text field.  Probably to verify that tge caret is
     within the bounds of the document.  
  */
  public int getLength()
  {
    /* Substract one from content length, since the last '\n' really
       isn't a part of the document, but is only there to distinguish
       the front position from the back position when the document is
       empty. */
    return content.length() - 1;
  }


  /* The caret listens to the document. Probably to see how changes in
     the document will affect the caret. BasicTextUI also adds
     listener. */
  public void addDocumentListener(DocumentListener listener)
  {
    listeners.add(listener);
  }

  public void removeDocumentListener(DocumentListener listener)
  {
    listeners.remove(listener);
  }

  public void addUndoableEditListener(UndoableEditListener listener)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public void removeUndoableEditListener(UndoableEditListener listener)
  {
    throw new UnsupportedOperationException("not implemented");
  }

  /* This method will be called by the view to retrieve properties
     such as "tabSize". */
  public Object getProperty(Object key)
  {
    return properties.get(key);
  }
  
  public void putProperty(Object key,
                        Object value)
  {
    if (value == null) 
      {
	properties.remove(key);
      }
    else
      {
	properties.put(key, value);
      }
  }


  /**
   * Remove is used to clear out the existing text before adding new text. 
   *
   *
   * @param length the number of characters to remove. The implementation
   * must be able to handle length=0.
   */
  public void remove(int offset, int length)
    throws BadLocationException
  {
    if (length == 0)
	return;

    content.remove(offset, length);

    DocumentEvent event = 
      new MyDocumentEvent(this, DocumentEvent.EventType.REMOVE,
			  offset, length);
    Iterator itr = listeners.iterator();
    while (itr.hasNext())
      {
	DocumentListener dl = (DocumentListener) itr.next();
	dl.removeUpdate(event);
      }

  }

  public void insertString(int offset,
			   String str,
			   AttributeSet as)
    throws BadLocationException
  {
    if (as != null)
      System.out.println("Ignoring attrubute set " + as + " during insert");
    content.insertString(offset, str);

    DocumentEvent event = 
      new MyDocumentEvent(this, DocumentEvent.EventType.INSERT,
			  offset, str.length());
    Iterator itr = listeners.iterator();
    while (itr.hasNext())
      {
	DocumentListener dl = (DocumentListener) itr.next();
	dl.insertUpdate(event);
      }
  }


  /* The view will retrieve portions of the text to display and
     measure optimal size. */
  public String getText(int offset,
			int length)
    throws BadLocationException
  {
    return content.getString(offset, length);
  }

  public void getText(int offset,
		      int length,
		      Segment txt)
    throws BadLocationException
  {
    content.getChars(offset, length, txt);
  }

  public Position getStartPosition()
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public Position getEndPosition()
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public Position createPosition(int offs)
    throws BadLocationException
  {
    return content.createPosition(offs);
  }

  public Element[] getRootElements()
  {
    throw new UnsupportedOperationException("not implemented");
  }


  /* Called by BasicTextUI as a result of a document model change. */
  public Element getDefaultRootElement()
  {
    return defaultRoot;
  }

  public void render(Runnable r)
  {
    throw new UnsupportedOperationException("not implemented");
  }
}

class MyElement implements Element
{
  Position start, end;
  Document document;
  List subElements = new ArrayList();

  public MyElement(Document document, int start, int end)
  {
    this.document = document;
    try
      {
	this.start = document.createPosition(start);
	this.end = document.createPosition(end);
      }
    catch(BadLocationException ble)
      {
	throw new RuntimeException(ble.toString());
      }
  }

  /* Views will ask the element what document it belongs to. */
  public Document getDocument()
  {
    return document;
  }

  public Element getParentElement()
  {
    throw new UnsupportedOperationException("not implemented");
  }

  public String getName()
  {
    throw new UnsupportedOperationException("not implemented");
  }

  /* This is called by the view during painting, to find out the appropriate
     appearance of this element. */
  public AttributeSet getAttributes()
  {
    //System.out.println(this+".getAttributes() ... returning null");
    return null;
  }


  /* This method is called by the view on child elements of the root
     element, to find the bounds of lines while calculating the
     longest line in a docuement. */
  public int getStartOffset()
  {
    return start.getOffset();
  }

  public int getEndOffset()
  {
    return end.getOffset();
  }

  public int getElementIndex(int offset)
  {
    for (int i=0; i<subElements.size(); i++)
      {
	Element el = (Element) subElements.get(i);
	if (el.getStartOffset() <= offset && 
	    el.getEndOffset() > offset)
	  return i;
      }
    return subElements.size()-1;
  }


  /* This will be called by the view as part of the longest line
     calculation. (Trying to find line elements?) */
  public int getElementCount()
  {
    return subElements.size();
  }

  public Element getElement(int index)
  {
    Element el = (Element) subElements.get(index);
    return el;
  }

  public boolean isLeaf()
  {
    return subElements.size() == 0;
  }
}

class Slider
{
  int offset;
  MyContent content;

  public Slider(int offset, MyContent mc)
  {
    this.offset = offset;
    this.content = mc;
  }

  public void dispose()
  {
    content.sliders.remove(this);
  }

  public String toString()
  {
    return Integer.toString(offset);
  }
}


class MyPosition implements Position
{
  Slider slider;

  public MyPosition(int offset, MyContent mc)
  {
    slider = new Slider(offset, mc);
  }

  public int getOffset()
  {
    return slider.offset;
  }

  protected void finalize()
  {
    slider.dispose();
  }

}


class MyContent implements Content
{
  
  List sliders;
  char[] string;
  int length;
  
  {
    sliders = new ArrayList();
    string = new char[4];
    string[0] = '\n';
    length = 1;
  }
  

  public Position createPosition(int offset)
    throws BadLocationException
  {
    MyPosition pos = new MyPosition(offset, this);
    sliders.add(pos.slider);
    return pos;
  }

  public int length()
  {
    return length;
  }
    
  public UndoableEdit insertString(int where, String insert)
    throws BadLocationException
  {

    int newLength = length + insert.length();

    char[] oldString = string;
    if (newLength > string.length)
      {
	string = new char[Math.max(newLength, string.length*2)];
	System.arraycopy(oldString, 0, string, 0, where);
      }

    System.arraycopy(oldString, where, string, where+insert.length(),
		     length-where);
    insert.getChars(0, insert.length(), string, where);
    length = newLength;
    
    slide(Math.max(1, where), insert.length());

    return null;
  }

  public UndoableEdit remove(int where, int nitems)
    throws BadLocationException
  {
    // Don't allow the trailing '\n' to be removed:
    if ((length-1) < (where+nitems))
      throw new BadLocationException("Cannot delete beyond end",
				     where+nitems);

    int tailStart = where+nitems;
    int tailLength = length - tailStart;

    System.arraycopy(string, tailStart, string, where, tailLength);
    length -= nitems;
    slide(where, -nitems);
    return null;
  }

  void slide(int minAffected, int delta) 
  {
    //System.out.println(this+".slide("+minAffected+", "+delta+")");
    //System.out.println("    was: " + sliders);
    Iterator itr = sliders.iterator();
    while (itr.hasNext())
      {
	Slider slider = (Slider) itr.next();
	if (slider.offset >= minAffected)
	  slider.offset = Math.max(minAffected, slider.offset + delta);
      }
    //System.out.println("    now: " + sliders);
  }

  public String getString(int where,
			  int len)
    throws BadLocationException
  {
    return new String(string, where, len);
  }

  public void getChars(int where,
		       int len,
		       Segment txt)
    throws BadLocationException
  {
    txt.array = string;
    txt.offset = where;
    txt.count = len;
  }

  public String toString() 
  {
    return "\"" + (new String(string, 0, length)) + "\"";
  }
}

class MyDocumentEvent implements DocumentEvent
{
  Document doc;
  EventType type;
  int offset;
  int length;

  public MyDocumentEvent(Document doc, EventType type, int offset, int length) 
  {
    this.doc = doc;
    this.type = type;
    this.offset = offset;
    this.length = length;
  }
  
  public int getOffset() 
  {
    return offset;
  }
  
  public int getLength() 
  {
    return length;
  }

  public Document getDocument() 
  {
    return doc;
  }
  
  public DocumentEvent.EventType getType()
  {
    return type;
  }
  
  public DocumentEvent.ElementChange getChange(Element elem)
  {
    return null;
  }
  
}

