
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;


public class MyFieldView extends MyView
{

  Segment tmpSegment = new Segment();


  public MyFieldView(Element element)
  {
    super(element);
  }

  
  public float getPreferredSpan(int axis)
  {
    try
      {
	switch (axis)
	  {
	  case X_AXIS:
	    return getTextWidth(getText(getLine()));
	  case Y_AXIS:
	    Container comp = getContainer();
	    FontMetrics fm = comp.getFontMetrics(comp.getFont());
	    return fm.getHeight();
	  }
	throw new IllegalArgumentException("invalid axis");
      }
    catch (BadLocationException ble)
      {
	throw new RuntimeException(ble.toString());
      }
  }
  
  /**
   * @returns bounds of edge at offset. Note, a null reference can be
   * returned.
   */
  public Shape modelToView(int pos, Shape a, Position.Bias b)
    throws BadLocationException
  {
    Rectangle bounds = a.getBounds();
    Element element = getElement();
    int start = element.getStartOffset();
    element.getDocument().getText(start, pos - start, tmpSegment);
    int x = bounds.x + getTextWidth(tmpSegment);
    int y = bounds.y;
    return new Rectangle(x, y, 1, getLineHeight());
  }
  
  public int viewToModel(float x, float y, Shape a, 
			 Position.Bias[] biasReturn)
    {
      Element element = getElement();
      Rectangle bounds = a.getBounds();
      Container comp = getContainer();
      FontMetrics fm = comp.getFontMetrics(comp.getFont());
      
      x -= bounds.x;
      y -= bounds.y;
      
      int start = element.getStartOffset();
      int end = Math.min(element.getEndOffset(), 
			 element.getDocument().getLength());
      float distance = Float.MAX_VALUE;
      int closest = start;
      try
        {
	  element.getDocument().getText(start, end - start, tmpSegment);
        }
      catch (BadLocationException ble)
        {
	  throw new RuntimeException(ble.toString());
        }
      
      for (int i = 0; i <= tmpSegment.count; i++)
        {
	  int w = fm.charsWidth(tmpSegment.array, tmpSegment.offset, i);
	  float d = Math.abs(x - w);
	  if (d < distance)
            {
	      distance = d;
	      closest = start + i;
            }
        }
      return closest;
    }
  
  
  public void paint(Graphics g, Shape allocation)
  {
    JTextComponent comp = (JTextComponent) getContainer();
    
    try
      {
	Caret caret = comp.getCaret();
	int dot = caret.getDot();
	int mark = caret.getMark();
	if (dot != mark)
	  {
	    g.setColor(comp.getSelectionColor());
	    Rectangle highlightArea = modelToView(dot, allocation,
						  Position.Bias.Forward).getBounds();
	    highlightArea.add(modelToView(mark, allocation,
					  Position.Bias.Forward).getBounds());
	    
	    g.fillRect(highlightArea.x, highlightArea.y,
		       highlightArea.width, highlightArea.height);
	  }
	Element line = getLine();
	Font font = comp.getFont();
	FontMetrics fm = comp.getFontMetrics(font);
	g.setFont(font);
	g.setColor(comp.getForeground());
	Rectangle bounds = allocation.getBounds();
	int x = bounds.x;
	int y = bounds.y + fm.getAscent();
	Segment txt = getText(line);
	//g.drawChars(txt.array, txt.offset, txt.count, x, y);
	if (mark > dot)
	  {
	    int tmp = mark;
	    mark = dot;
	    dot = tmp;
	  }
	if (txt.offset < mark)
	  {
	    g.setColor(comp.getForeground());
	    paint(g, x, y, txt, txt.offset, mark);
	  }
	if (mark < dot)
	  {
	    g.setColor(comp.getSelectedTextColor());
	    paint(g, x, y, txt, mark, dot);
	  }
	if (dot < (txt.offset + txt.count))
	  {
	    g.setColor(comp.getForeground());
	    paint(g, x, y, txt, dot, txt.offset + txt.count);
	  }
      }
    catch (BadLocationException ble)
      {
	System.err.println(ble);
      }
  }
  
  void paint(Graphics g, int x, int y, Segment txt, int s, int e) {
    if (txt.offset != s)
      {
	Container comp = getContainer();
	FontMetrics fm = comp.getFontMetrics(comp.getFont());
	x += fm.charsWidth(txt.array, txt.offset, s - txt.offset);
      }
    g.drawChars(txt.array, s, e - s, x, y);
  }
  
  
  // -------- Helper methods:
  
  final int getLineHeight()
  {
    Container comp = getContainer();
    FontMetrics fm = comp.getFontMetrics(comp.getFont());
    return fm.getHeight();
  }
  
  
  final Element getLine()
  {
    return getElement().getElement(0);
  }
  
  final Segment getText(Element element) throws BadLocationException
  {
    int start = element.getStartOffset();
    int end = Math.min(element.getEndOffset(), 
		       element.getDocument().getLength());
    
    element.getDocument().getText(start, end - start, tmpSegment);
    return tmpSegment;
  }
  
  
  final int getTextWidth(Segment txt)
  {
    Container comp = getContainer();
    FontMetrics fm = comp.getFontMetrics(comp.getFont());
    return fm.charsWidth(txt.array, txt.offset, txt.count);
  }
  
  
  public void insertUpdate(DocumentEvent e, Shape a, ViewFactory f)
  {
    repaintWholeLine(a);
  }
  
  public void removeUpdate(DocumentEvent e, Shape a, ViewFactory f)
  {
    repaintWholeLine(a);
  }
  
  public void changedUpdate(DocumentEvent e, Shape a, ViewFactory f)
  {
    repaintWholeLine(a);
  }

  final void repaintWholeLine(Shape a)
  {
    // FIXME: do a little more nuanced repainting
    Component comp = getContainer();
    Rectangle area = a.getBounds();
    comp.repaint(area.x, area.y,
		 area.width, Math.min(area.height, getLineHeight()));
  }
}

