
import java.awt.*;
import java.awt.geom.*;
import javax.swing.event.*;
import javax.swing.event.DocumentEvent.*;

import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.text.*;
import javax.swing.text.AbstractDocument.Content;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.Element;
import javax.swing.text.BadLocationException;
import javax.swing.text.Segment;
import javax.swing.text.Position;
import javax.swing.text.AttributeSet;
import javax.swing.undo.*;
import java.util.*;


public class MyTextUI extends TextUI
{
  JTextComponent component;
  BindView view;

  protected Caret createCaret()
  {
    Caret caret = new MyCaret();
    //caret.setBlinkRate(500);
    return caret;
  }

  protected Highlighter createHighlighter()
  {
    return new MyDefaultHighlighter();
  }

  //protected String getKeymapName()

  protected Keymap createKeymap()
  {
    return JTextComponent.getKeymap(JTextComponent.DEFAULT_KEYMAP);
  }

  //protected void propertyChange(PropertyChangeEvent evt);

  //protected abstract String getPropertyPrefix();
  
  protected void installDefaults() 
  {
    if (component.getCaret() == null) 
	component.setCaret(createCaret());
    
    if (component.getHighlighter() == null)
	component.setHighlighter(createHighlighter());

    // font
    component.setBackground(Color.white);
    component.setForeground(Color.black);
    component.setCaretColor(Color.black);
    component.setSelectionColor(new Color(0x54, 0x70, 0x98));
    component.setSelectedTextColor(Color.white);
    
    if (component.getKeymap() == null)
	component.setKeymap(createKeymap());
  }

  protected void uninstallDefaults() 
  {
  }

  protected void installListeners()
  {
    component.getDocument().addDocumentListener(view);
  }

  protected void uninstallListeners()
  {
  }

  protected void installKeyboardActions()
  {
  }

  protected void uninstallKeyboardActions()
  {
  }

  protected void paintBackground(Graphics gfx)
  {
    Dimension dim = component.getSize();
    gfx.setColor(component.getBackground());
    gfx.fillRect(0, 0, dim.width, dim.height);
  }





  public void damageRange(JTextComponent comp, int start, int end)
  {
    try 
      {
	Rectangle area = modelToView(comp, start, Position.Bias.Forward);
	if (area == null)
	  {
	    System.out.println("ignored null area returned by modelToView");
	    return;
	  }

	area.add(modelToView(comp, end, Position.Bias.Backward));
	comp.repaint(area);
      }
    catch (BadLocationException ble)
      {
	throw new RuntimeException(ble.toString());
      }
  }
  public void damageRange(JTextComponent comp, int i, int i2, Position.Bias positionBias, Position.Bias p)
  {
    System.out.println("*** UNIMPLEMENTED ***");
    throw new UnsupportedOperationException("not implemented");
  }





  /** The editor kit is used to get factory for creating views. */

  EditorKit editorKit = new DefaultEditorKit();

  public EditorKit getEditorKit(JTextComponent comp)
  {
    return editorKit;
  }

  public int getNextVisualPositionFrom(JTextComponent comp, int pos, 
				       Position.Bias bias, int direction, 
				       Position.Bias[] biasReturn)
    throws BadLocationException
  {
    return view.getNextVisualPositionFrom(pos, bias, direction, biasReturn);
  }

  public View getRootView(JTextComponent comp)
  {
    System.out.println("*** UNIMPLEMENTED ***");
    throw new UnsupportedOperationException("not implemented");
  }

  /**
   * @returns bounds of edge at offset. Note, a null reference can be
   * returned.
   */
  public Rectangle modelToView(JTextComponent comp, int offset)
     throws BadLocationException
  {
    return view.modelToView(offset, view.getRegion(),
			    Position.Bias.Forward).getBounds();
  }
  /**
   *
   * This method is typically called by the caret to find out where to draw.
   */
  public Rectangle modelToView(JTextComponent comp, int offset,
			       Position.Bias bias) 
    throws BadLocationException
  {
    return view.modelToView(offset, view.getRegion(), bias).getBounds();
    
  }
  public int viewToModel(JTextComponent comp, Point point)
  {
    System.out.println("*** SHORT VERSION UNIMPLEMENTED ***");
    throw new UnsupportedOperationException("not implemented");
  }
  /** 
   * This method will be called during positioning of the
   * caret. Typically will DefaultCaret listen to mouse presses on the
   * component and call DefaultCaret.positionCaret() as a result of a
   * mouse click. viewToModel will in turn be called to translate the
   * point from the mouse event to an offset in the docuement model.
   */
  public int viewToModel(JTextComponent comp, Point point,
			 Position.Bias[] biasReturn)
  {
    return view.viewToModel(point.x, point.y, view.getRegion(), biasReturn);
  }

  /*
    installUI should call this.modelChanged(...)
    ...should call $RootView.getViewFactory(...) ?
    ...should call this.getEditorKit(...)
  */
  public void installUI(JComponent comp) 
  {
    System.out.println("installing my text ui");

    component = (JTextComponent) comp;

    Element element = component.getDocument().getDefaultRootElement();
    MyView eview = new MyFieldView(element);
    view = new BindView(eview, comp);

    installDefaults();
    installListeners();
    installKeyboardActions();
  }

  public void uninstallUI(JComponent c)
  {
    uninstallDefaults();
    uninstallListeners();
    uninstallKeyboardActions();
    view = null;
    component = null;
  }

  public final void paint(Graphics g, JComponent c)
  {
    // FIXME: assert read lock
    paintSafely(g);
    // FIXME: release read lock
  }

  public void paintSafely(Graphics gfx)
  {
    paintBackground(gfx);

    Caret caret = component.getCaret();
    
    Highlighter h = component.getHighlighter();
    if (h != null)
      {
	/* This has no effect if the highlighter is of type
	   LayeredHighlighter. In this case, the highlight is drawn as
	   a part of the view. */
	h.paint(gfx);
      }

    Shape region = view.getRegion();
    Rectangle2D bounds = view.getRegion().getBounds2D();
    
    view.setSize((float) bounds.getWidth(), (float) bounds.getHeight());

    view.paint(gfx, region);

    if (caret != null)
      caret.paint(gfx);
  }

  public Dimension getPreferredSize(JComponent c)
  {
    float w = view.getPreferredSpan(View.X_AXIS);
    float h = view.getPreferredSpan(View.Y_AXIS);
    Dimension dim = new Dimension((int) Math.ceil(w), (int) Math.ceil(h));
    return dim;
  }
}


/** A decorator view that binds the decorated view to a specific
    component.  This basically involves implementing getComponent() to
    point to the desired component, and forwarding the rest of the
    requests to the decorated view. */
final class BindView extends MyView implements DocumentListener 
{
  MyView decorated;
  Container container;
  
  public BindView(MyView decorated, Container container) 
  {
    super(decorated.getElement());
    this.decorated = decorated;
    this.container = container;
    decorated.setParent(this);
  }

  public Container getContainer() 
  {
    return container;
  }

  public float getPreferredSpan(int axis)
  {
    return decorated.getPreferredSpan(axis);
  }

  public void paint(Graphics g, Shape allocation)
  {
    decorated.paint(g, allocation);
  }

  public void setSize(float width, float height) 
  {
    decorated.setSize(width, height);
  }

  public Shape modelToView(int pos, Shape a, Position.Bias b)
    throws BadLocationException 
  {
    return decorated.modelToView(pos, a, b);
  }

  public int viewToModel(float x, float y, Shape a,
			 Position.Bias[] biasReturn) 
  {
    return decorated.viewToModel(x, y, a, biasReturn);
  }

  public void insertUpdate(DocumentEvent e) 
  {
    decorated.insertUpdate(e, getRegion(), null);
  }

  public void removeUpdate(DocumentEvent e)
  {
    decorated.removeUpdate(e, getRegion(), null);
  }

  public void changedUpdate(DocumentEvent e)
  {
    decorated.changedUpdate(e, getRegion(), null);
  }

  Shape getRegion() 
  {
    Rectangle rect = container.getBounds();
    Insets insets = container.getInsets();
    rect.x = insets.left;
    rect.y = insets.top;
    rect.width -= insets.left + insets.right;
    rect.height -= insets.top + insets.bottom;
    return rect;
}

  public int getNextVisualPositionFrom(int pos, Position.Bias bias,
				       int direction,
				       Position.Bias[] biasReturn)
    throws BadLocationException
  {
    return decorated.getNextVisualPositionFrom(pos, bias, getRegion(),
					       direction, biasReturn);
  }
}
