/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

package javax.swing;

import java.io.Serializable;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.Image;
import java.awt.Graphics;
import java.net.URL;
//import java.awt.MediaTracker;

public class ImageIcon implements Icon, Serializable
{
  protected static final Component component;
  
  Image image;
  
  String description;
  
  //protected static final java.awt.MediaTracker tracker;
  
  public ImageIcon(String filename, String description)
  {
    this(Toolkit.getDefaultToolkit().getImage(filename), description);
  }
    
  public ImageIcon(String filename)
  {
    this(Toolkit.getDefaultToolkit().getImage(filename));
  }
  
  public ImageIcon(URL location, String description)
  {
    this(Toolkit.getDefaultToolkit().getImage(location), description);
  }

  public ImageIcon(URL location)
  {
    this(Toolkit.getDefaultToolkit().getImage(location));
  }
    
  public ImageIcon(Image image, String description)
  {
    this(image);
    setDescription(description);
  }
    
  public ImageIcon(Image image)
  {
    setImage(image);
  }
    
  public ImageIcon(byte[] imageData, String description)
  {
    this(Toolkit.getDefaultToolkit().createImage(imageData), description);
  }    

  public ImageIcon(byte[] imageData)
  {
    this(Toolkit.getDefaultToolkit().createImage(imageData));
  }

  public ImageIcon() {}

  //protected void loadImage(Image image)
  
  //public int getImageLoadStatus()

  public Image getImage()
  {
    return image;
  }    

  public void setImage(Image image)
  {
    this.image = image;
  }
    
  public String getDescription()
  {
    return description;
  }
    
  public void setDescription(String description)
  {
    this.description = description;
  }

  public void paintIcon(Component component, Graphics gfx, int x, int y)
  {
    gfx.drawImage(image, x, y, component);
  }
  
  public int getIconWidth()
  {
    return image.getWidth(null);
  }

  public int getIconHeight()
  {
    return image.getHeight(null);
  }
    
  //public void setImageObserver(ImageObserver observer);
  
  //public ImageObserver getImageObserver();

  public String toString()
  {
    return getClass().getName() + "[image=" + image +
      (description != null ? ("," + description) : "") + "]";
  }
  
  //public AccessibleContext getAccessibleContext();
}
