/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

package javax.swing;

import java.util.*;
import java.lang.reflect.*;
import javax.swing.plaf.ComponentUI;

/**
 * Status: minimal skeleton
 */
public class UIDefaults extends Hashtable
{
  static final Class[] CREATE_UI_PARAM = { JComponent.class };
  
  public UIDefaults(Object[] keyVal)
  {
    for (int i=0; i<keyVal.length;)
      {
	Object key   = keyVal[i++];
	Object value = keyVal[i++];
	put(key, value);
      }
  }

  public ComponentUI getUI(JComponent target)
  {
    Class uiclass = getUIClass(target.getUIClassID());
    try
      {
	Method createMethod =
	  uiclass.getMethod("createUI", CREATE_UI_PARAM);
	Object[] createArg = { target };
	return (ComponentUI) createMethod.invoke(createMethod, createArg);
      }
    catch (InvocationTargetException e)
      {
	throw new RuntimeException(e.toString());
      }
    catch (NoSuchMethodException e)
      {
	throw new RuntimeException(e.toString());
      } 
    catch (IllegalAccessException e) 
      {
	throw new RuntimeException(e.toString());
      }
  }

  public String getString(Object key)
  {
    Object val = get(key);
    if (val instanceof String)
      return (String) val;
    return null;
  }
    
  public Class getUIClass(String uiClassID)
  {
    String className = getString(uiClassID);
    
    ClassLoader cl = ClassLoader.getSystemClassLoader();
    try 
      {
	return cl.loadClass(className);
      } 
    catch (ClassNotFoundException e) 
      {
	throw new RuntimeException(e.toString());
      } 
    catch (InstantiationException e) 
      {
	throw new RuntimeException(e.toString());
      } 
    catch (IllegalAccessException e) 
      {
	throw new RuntimeException(e.toString());
      }	
  }
}
