/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

package javax.swing.border;

import java.awt.*;

public class BevelBorder extends AbstractBorder
{

  public static final int RAISED = 0,
                          LOWERED = 1;

  protected int bevelType;
  protected Color highlightOuter;
  protected Color highlightInner;
  protected Color shadowInner;
  protected Color shadowOuter;
  
  public BevelBorder(int bevelType) 
  {
    this.bevelType = bevelType;
  }

  public BevelBorder(int bevelType, Color highlight, Color shadow) 
  {
    this(bevelType,
	 highlight.brighter(), highlight,
	 shadow, shadow.darker());
  }	
  
  public BevelBorder(int bevelType,
		     Color highlightOuter, Color highlightInner,
		     Color shadowOuter, Color shadowInner) 
  {
    this(bevelType);
    this.highlightOuter = highlightOuter;
    this.highlightInner = highlightInner;
    this.shadowOuter = shadowOuter;
    this.shadowInner = shadowInner;
  }

  public void paintBorder(Component c, Graphics gfx,
			  int x, int y, int width, int height) 
  {
    if (bevelType == RAISED) 
      {
	paintRaisedBevel(c, gfx, x, y, width, height);
      } 
    else 
      {
	paintLoweredBevel(c, gfx, x, y, width, height);
      }
  }

  public Insets getBorderInsets(Component c) 
  {
    // TODO: check whether cloning is preferable.
    return (Insets) INSETS_2_PROTOTYPE.clone();
  }

  public Insets getBorderInsets(Component c, Insets insets) 
  {
    insets.top    = 2;
    insets.left   = 2;
    insets.bottom = 2;
    insets.right  = 2;
    return insets;
  }

  public Color getHighlightOuterColor(Component c) 
  {
    if (highlightOuter != null)
      return highlightOuter;
    return c.getBackground().brighter().brighter();
  }
    
  public Color getHighlightInnerColor(Component c)
  {
    if (highlightInner != null) 
      return highlightInner;
    return c.getBackground().brighter();
  }
  
  public Color getShadowOuterColor(Component c) 
  {
    if (shadowOuter != null) 
      return shadowOuter;
    return c.getBackground().darker().darker();
  }
    
  public Color getShadowInnerColor(Component c) 
  {
    if (shadowInner != null) 
      return shadowInner;
    return c.getBackground().darker();
  }

  public Color getHighlightOuterColor() 
  {
    return highlightOuter;
  }
    
  public Color getHighlightInnerColor() 
  {
    return highlightInner;
  }

  public Color getShadowOuterColor() 
  {
    return shadowOuter;
  }
    
  public Color getShadowInnerColor() 
  {
    return shadowInner;
  }

  public int getBevelType() 
  {
    return bevelType;
  }
    
  public boolean isBorderOpaque() 
  {
    return true;
  }
    
  protected void paintRaisedBevel(Component c, Graphics gfx,
				  int x, int y, int width, int height) 
  {
    Color tlo = getHighlightOuterColor(c);
    Color tli = getHighlightInnerColor(c);
    Color bro = getShadowOuterColor(c);
    Color bri = getShadowInnerColor(c);
    
    int xr = x + width  - 1;
    int yb = y + height - 1;
    
    paint3DBorder(gfx, x,   y,   xr,   yb,   tlo, bro, 0, 1);
    paint3DBorder(gfx, x+1, y+1, xr-1, yb-1, tli, bri, 0, 1);
  }
    
  protected void paintLoweredBevel(Component c, Graphics gfx,
				   int x, int y, int width, int height) 
  {
    Color bro = getHighlightOuterColor(c);
    Color bri = getHighlightInnerColor(c);
    Color tlo = getShadowOuterColor(c);
    Color tli = getShadowInnerColor(c);
    
    int xr = x + width  - 1;
    int yb = y + height - 1;
    
    paint3DBorder(gfx, x,   y,   xr,   yb,   tlo, bro, 0, 1);
    paint3DBorder(gfx, x+1, y+1, xr-1, yb-1, tli, bri, 0, 1);
  }
}
