/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

package javax.swing.border;

import java.awt.*;

public class EtchedBorder extends AbstractBorder 
{
  public static final int RAISED  = 0;
  public static final int LOWERED = 1;
  protected int etchType;
  protected Color highlight;
  protected Color shadow;
  
  public EtchedBorder() 
  {
    this(LOWERED);
  }
  
  public EtchedBorder(int etchType) 
  {
    this(etchType, null, null);
  }
  
  public EtchedBorder(Color highlight, Color shadow) 
  {
    this(RAISED, highlight, shadow);
  }

  public EtchedBorder(int etchType, Color highlight, Color shadow) 
  {
    this.etchType = etchType;
    this.highlight = highlight;
    this.shadow = shadow;
  }
  
  public void paintBorder(Component c, Graphics gfx,
			  int x, int y, int width, int height) 
  {
    Color tl, br;
    if (etchType == RAISED) 
      {
	tl = getHighlightColor(c);
	br = getShadowColor(c);
      } 
    else 
      {
	br = getHighlightColor(c);
	tl = getShadowColor(c);
      }
    
    int xr = x + width  - 1;
    int yb = y + height - 1;	
    
    paint3DBorder(gfx, x,   y,   xr,   yb,   tl, br, 1, 1);
    paint3DBorder(gfx, x+1, y+1, xr-1, yb-1, br, tl, 1, 1);
  }
    
  public Insets getBorderInsets(Component c) 
  {
    // TODO: check whether cloning is preferable.
    return (Insets) INSETS_2_PROTOTYPE.clone();
  }

  public Insets getBorderInsets(Component c, Insets insets) 
  {
    insets.top    = 2;
    insets.left   = 2;
    insets.bottom = 2;
    insets.right  = 2;
    return insets;
  }
    
  public boolean isBorderOpaque() 
  {
    return true;
  }
  
  public int getEtchType() 
  {
    return etchType;
  }
  
  public Color getHighlightColor(Component c) 
  {
    if (highlight != null) 
      return highlight;
    return c.getBackground().brighter();
  }
  
  public Color getHighlightColor() 
  {
    return highlight;
  }
    
  public Color getShadowColor(Component c) 
  {
    if (shadow != null) 
      return shadow;
    return c.getBackground().darker();
  }

  public Color getShadowColor() 
  {
    return shadow;
  }
}
