/* Copyright (C) 2000  Free Software Foundation

   This file is part of libgcj.

This software is copyrighted work licensed under the terms of the
Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
details.  */

package javax.swing.plaf;

import javax.swing.*;
import java.awt.*;

public abstract class ComponentUI 
{
  public ComponentUI() 
  {
  }
  
  public void installUI(JComponent c) 
  {
  }
  
  public void uninstallUI(JComponent c) 
  {
  }
  
  public void update(Graphics gfx, JComponent c) 
  {
    // Assume clipping had been set adequately
    
    gfx.setColor(c.getBackground());
    gfx.fillRect(0, 0, c.getWidth()+200, c.getHeight()+200);
    
    paint(gfx, c);
  }
  
  /**
   * Draw the UI imagery of the component.  Coordinate space is
   * relative to the top-left corner of the component.  Remember to
   * reserve space for the component border by reading the insets of
   * the component, and translating the coordinates accordingly.
   *
   * <p>Default implementation does nothing.  Remember to set correct
   * color befor starting to paint.  The update() method may have set
   * the background color, which will result in some very boring
   * imagery unless the color is changed.
   */
  public void paint(Graphics gfx, JComponent c) 
  {
  }
  
  /**
   * Calculate and return the preferred size of the
   * component. Remember to include the insets of the component in the
   * calculation. 
   */
  public Dimension getPreferredSize(JComponent c) 
  {
    return c.getSize();
  }
  
  public Dimension getMinimumSize(JComponent c) 
  {
    return getPreferredSize(c);
  }
    
  public Dimension getMaximumSize(JComponent c) 
  {
    return getPreferredSize(c);
  }

  /**
   * See documentation for java.awt.Component.contains().  Remember
   * that x and y is specified in the coordinate system of the
   * component.  
   */
  public boolean contains(JComponent c, int x, int y) 
  {
    return (x>=0) && (y>=0) && (x<c.getWidth()) && (y<c.getHeight());
  }
  
  public static ComponentUI createUI(JComponent c) 
  {
    throw new UnsupportedOperationException("not implemented yet");	
  }
    
  public int getAccessibleChildrenCount(JComponent c) 
  {
    throw new UnsupportedOperationException("not implemented yet");
  }
  
  //public Accessible getAccessibleChild(JComponent c, int i);
}
